/*-
 * ~~~~~~licensing~~~~~~
 * profile-manager
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.management.profile.manager.mapper;

import org.mapstruct.InjectionStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Mappings;
import ru.entaxy.esb.system.jpa.entity.ExportAllowed;
import ru.entaxy.esb.system.jpa.entity.System;
import ru.entaxy.esb.system.management.bridge.profile.manager.dto.BridgeProfileDto;
import ru.entaxy.esb.system.management.bundle.jpa.dto.BundleDto;
import ru.entaxy.esb.system.management.bundle.jpa.entity.BundleEntity;
import ru.entaxy.esb.system.management.connector.manager.mapper.ConnectorMapper;
import ru.entaxy.esb.system.management.profile.manager.dto.ProfileDto;

import java.util.List;
import java.util.Set;

@Mapper(componentModel = "spring", uses = {ConnectorMapper.class},
        injectionStrategy = InjectionStrategy.CONSTRUCTOR)
public interface ProfileMapper {

    @Mappings({
            @Mapping(target = "version", source = "version", qualifiedByName = "generateVersion"),
            @Mapping(target = "createDate", expression = "java(new java.util.Date())"),
    })
    System toSystemEntity(ProfileDto profileDto);

    @Mappings({
            @Mapping(target = "status", source = "bundleEntity.status"),
    })
    ProfileDto toProfileDto(System system);

    @Mappings({
            @Mapping(target = "name", source = "bundleName"),
            @Mapping(target = "url", source = "bundleUrl"),
    })
    BundleEntity toBundleEntity(BundleDto bundle);
    
    @Mappings({
        @Mapping(target = "bundleName", source = "name"),
        @Mapping(target = "bundleUrl", source = "url"),
    })
    BundleDto toBundleDto(BundleEntity bundle);

    Set<ExportAllowed> toExportAllowedEntities(Set<String> exportAllowed);

    Set<String> toExportAllowed(Set<ExportAllowed> exportAllowed);

    default ExportAllowed map(String exportAllowed) {
        ExportAllowed exportAllowedEntity = new ExportAllowed();
        exportAllowedEntity.setName(exportAllowed);
        return exportAllowedEntity;
    }

    default String map(ExportAllowed exportAllowed) {
        return exportAllowed != null ? exportAllowed.getName() : null;
    }

    List<ProfileDto> toListProfiles(List<System> systemList);

    @Mappings({
            @Mapping(target = "profileUuid", source = "uuid")
    })
    BridgeProfileDto toBridgeProfileDto(System system);
}
